/* gcc -o cursor-manager `pkg-config --cflags --libs gtk+-3.0 gtk+-x11-3.0 xi xfixes` cursor-manager.c */
#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <X11/extensions/XInput2.h>
#include <X11/extensions/Xfixes.h>

static gboolean cursor_shown = TRUE;

static GdkFilterReturn
raw_event_filter (GdkXEvent *xevent,
                  GdkEvent  *event,
                  gpointer   user_data)
{
  GdkDisplay *display;
  GdkScreen *screen;
  XEvent *ev = xevent;
  XIEvent *xiev;

  if (ev->type != GenericEvent)
    return GDK_FILTER_CONTINUE;

  xiev = ev->xcookie.data;
  display = gdk_display_get_default ();
  screen = gdk_display_get_default_screen (display);

  switch (xiev->evtype)
    {
    case XI_RawTouchBegin:
    case XI_RawTouchUpdate:
      if (cursor_shown)
        {
          XFixesHideCursor (GDK_DISPLAY_XDISPLAY (display),
                            GDK_WINDOW_XID (gdk_screen_get_root_window (screen)));
          cursor_shown = FALSE;
        }
      break;
    case XI_RawButtonPress:
    case XI_RawButtonRelease:
    case XI_RawMotion:
      if (!cursor_shown)
        {
          XFixesShowCursor (GDK_DISPLAY_XDISPLAY (display),
                            GDK_WINDOW_XID (gdk_screen_get_root_window (screen)));
          cursor_shown = TRUE;
        }
      break;
    default:
      break;
    }

  return GDK_FILTER_CONTINUE;
}

int
main (int argc, char *argv[])
{
  GdkRGBA transparent = { 0 };
  XIEventMask mask;
  guchar *evmask;
  GtkWidget *window;
  GdkScreen *screen;
  cairo_region_t *region;
  GdkCursor *cursor;
  Display *xdisplay;
  GdkDisplay *display;

  gtk_init (NULL, NULL);

  screen = gdk_screen_get_default ();

  evmask = g_new0 (guchar, XIMaskLen (XI_LASTEVENT));
  XISetMask (evmask, XI_RawTouchBegin);
  XISetMask (evmask, XI_RawTouchEnd);
  XISetMask (evmask, XI_RawTouchUpdate);
  XISetMask (evmask, XI_RawButtonPress);
  XISetMask (evmask, XI_RawButtonRelease);
  XISetMask (evmask, XI_RawMotion);

  mask.deviceid = XIAllMasterDevices;
  mask.mask_len = sizeof (evmask);
  mask.mask = evmask;

  XISelectEvents (GDK_SCREEN_XDISPLAY (screen),
                  GDK_WINDOW_XID (gdk_screen_get_root_window (screen)),
                  &mask, 1);

  gdk_window_add_filter (NULL, raw_event_filter, window);

  gtk_main ();
}
